#include "cora_fit.h"

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

int check_bins()
{
  int i,nb=0,j=1,k=1,tcount, newcount;
  double dbin=0, tolbin=1.6, mbin;

Vector newwave;    /* wavelength vector    */
Vector newsp;      /* count vector         */
Vector newerr;     /* count errors         */
Vector newbkgd;    /* instr. background    */
Vector newbkerr;   /* error in background  */
Vector newbg0;     /* original background  */

  vec_init(&newwave);
  vec_init(&newsp);
  vec_init(&newerr);
  vec_init(&newbkgd);
  vec_init(&newbkerr);
  vec_init(&newbg0);

  tcount=wave.len;

  for (i=1;i<tcount;++i)
   {
     dbin = wave.vec[i]-wave.vec[i-1];
     if (dbin > tolbin*binsize) ++nb;
   }

 if (nb > 0)
 {
  fprintf(stderr,"Your spectrum is not well binned. You have the option to fix this internally.\n");
  fprintf(stderr," - The program assumes the bins roughly equidistant (to within %4.2f%% )\n",(tolbin-1.)*100.);
  fprintf(stderr," - If the flag fill_empty_bins is set to 1 (is %d now), empty bins will be filled with zeros. Otherwise the program will assume the last bin before a bin gap to be as wide as the binsize plus the whole gap\n",fixflag);
  fprintf(stderr," NOTE: Your spectrum will not be changed, all changes are only made internally.\n");

  newcount = tcount + nb;
  vec_alloc(&newwave,newcount);
  vec_alloc(&newsp,newcount);
  vec_alloc(&newerr,newcount);
  vec_alloc(&newbkgd,newcount);
  vec_alloc(&newbkerr,newcount);
  vec_alloc(&newbg0,newcount);

  newwave.vec[0] = wave.vec[0];
  newsp.vec[0] = sp.vec[0];
  newerr.vec[0] = err.vec[0];
  newbkgd.vec[0] = bkgd.vec[0];
  newbkerr.vec[0] = bkerr.vec[0];
  newbg0.vec[0] = bg0.vec[0];

i=1;
j=1;
while(i < newcount)
  {
   while (wave.vec[j]-wave.vec[j-1] <= tolbin*binsize && i < newcount)
      {
        newwave.vec[i] = wave.vec[j];
        newsp.vec[i] = sp.vec[j];
        newerr.vec[i] = err.vec[j];
        newbkgd.vec[i] = bkgd.vec[j];
        newbkerr.vec[i] = bkerr.vec[j];
        newbg0.vec[i] = bg0.vec[j];
        ++j;++i;
      }
   k=1;
   dbin = (wave.vec[j+1]-wave.vec[j-1]);
   mbin = dbin / ceil(dbin/binsize);
   while (wave.vec[j-1]+(k-1)*mbin <= wave.vec[j] && i < newcount)
      {
        newwave.vec[i] = wave.vec[j-1]+k*mbin;
        newsp.vec[i] = 0.;/*bkgd.vec[j]; */
        newerr.vec[i] = newsp.vec[i];
        newbkgd.vec[i] = newsp.vec[i];
        newbkerr.vec[i] = newbkgd.vec[i];
        newbg0.vec[i] = newbkgd.vec[i];
        ++i;++k;
      }
   ++j;
   }

   if (fixflag > 0)
    {
      vec_free(&wave);
      vec_free(&sp);
      vec_free(&err);
      vec_free(&bkgd);
      vec_free(&bkerr);
      vec_free(&bg0);

      vec_init(&wave);
      vec_init(&sp);
      vec_init(&err);
      vec_init(&bkgd);
      vec_init(&bkerr);
      vec_init(&bg0);

      vec_alloc(&wave,newcount);
      vec_alloc(&sp,newcount);
      vec_alloc(&err,newcount);
      vec_alloc(&bkgd,newcount);
      vec_alloc(&bkerr,newcount);
      vec_alloc(&bg0,newcount);
      for (i=0;i<newcount;++i)
       {
	wave.vec[i] = newwave.vec[i];
	sp.vec[i] = newsp.vec[i];
	err.vec[i] = newerr.vec[i];
	bkgd.vec[i] = newbkgd.vec[i];
	bkerr.vec[i] = newbkerr.vec[i];
	bg0.vec[i] = newbg0.vec[i];
       }
     binsize = (wave.vec[wave.len-1] - wave.vec[0]) / (wave.len-1);
    }
 }
  return (nb);
} 
