#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "config.h"

#define MAXTEXTL 128
#define MAXPATH  512

#ifdef WITH_CFITSIO

#include "fitsio.h"

float *wave, *sp, *err, *bg, *erbg, *aeff;
double alp=2.750730142e-2,d=15488.455078125,m=1.;
double dbet=1.208e-5,betref=0.03578524;
double dt=1.,exposure;

char title[MAXTEXTL];
char tabtit[MAXTEXTL];
char rname[MAXTEXTL];
int ndim=30,i=0,*lnum;
static double hc=12.415155;
static int lamcor=0.;

int fitscopy(char *argv, char *outfile);

void printerror( int status);

void terminate(int n)
{

  if (n != 0) exit(n);
  return;

}

/*  -------------------------------------------------------------------- */

int read_rgs (char *path, char *bpath, char *rmfpath, char *totpath, char *apath)
{

  fitsfile *fd, *fd2, *fd3;
  int status, nfound, anynull, ii, rmfflag=0, totflag=0;
  int frow, felem, nelem, intnull, *chan, *qual, *bqual, *good;
  double doublenull, *rate, *brate, *staterr, *berr, *flwave, *flux, *dlam, *e_min, *e_max;
  char message[BUFSIZ], *ttype[4];
  long nrow;

  char dummy[BUFSIZ];

/*   declarations for analyzing flux file flux.txt (calculating aeff)
  char line[BUFSIZ];
  int items;
  FILE * fda; */

  double calp,bet;

  status = 0;
  calp=cos(alp);

  if ( 0 == access(totpath, R_OK) )
    {
     fits_open_file(&fd, totpath, READONLY, &status);
     totflag=1;
    }
  else
     if ( 0 == access(path, R_OK) )
	fits_open_file(&fd, path, READONLY, &status);
     else {fprintf(stderr,"no spectrum file available, neither %s nor %s\n",totpath,path);
	terminate(8);}
  if ( fits_open_file(&fd2, bpath, READONLY, &status) )
	printerror( status );

  if ( fits_movnam_hdu(fd, BINARY_TBL, "SPECTRUM", 0, &status) )
	printerror( status );
  if ( fits_movnam_hdu(fd2, BINARY_TBL, "SPECTRUM", 0, &status) )
        printerror( status );

/* --------------------------------------- */
/*  determine important header information */
/* --------------------------------------- */
   snprintf(message, BUFSIZ,"Files %s and %s do not match: different",path,bpath);

/*  determine number of rows */
  if (fits_read_key (fd, TLONG, "NAXIS2", &nrow, dummy, &status))
      printerror( status ); 
  ndim=nrow;
  if (fits_read_key (fd2, TLONG, "NAXIS2", &nrow, dummy, &status))
      printerror( status );
  if (nrow != ndim) {fprintf(stderr,"%s %s\n 
      Source: %d Background: %ld\n",message,dummy,ndim,nrow);terminate(8);}
/*  determine exposure time */
  if (fits_read_key (fd, TDOUBLE, "EXPOSURE", &doublenull, dummy, &status))
      printerror( status );
  dt=doublenull;
  if (fits_read_key (fd2, TDOUBLE, "EXPOSURE", &doublenull, dummy, &status))
      printerror( status );
  if (dt != doublenull) {fprintf(stderr,"%s %s\n
      Source: %e Background: %e\n",message,dummy,dt,doublenull);terminate(8);}
  exposure=dt;

/*  determine midpoint of bin at origin  */
  if (fits_read_key (fd, TDOUBLE, "TCRVL1", &doublenull, dummy, &status))
      printerror( status );
  betref=doublenull;
  if (fits_read_key (fd2, TDOUBLE, "TCRVL1", &doublenull, dummy, &status))
      printerror( status );
  if (betref != doublenull) {fprintf(stderr,"%s %s\n
      Source: %e Background: %e\n",message,dummy,betref,doublenull);terminate(8);}

/*  determine uniform bin width  */
  if (fits_read_key (fd, TDOUBLE, "TCDLT1", &doublenull, dummy, &status))
      printerror( status );
  dbet=doublenull;
  if (fits_read_key (fd2, TDOUBLE, "TCDLT1", &doublenull, dummy, &status))
      printerror( status );
  if (dbet != doublenull) {fprintf(stderr,"%s %s\n
      Source: %e Background: %e\n",message,dummy,dbet,doublenull);terminate(8);}

/*  determine Alpha  */
  if (fits_read_key (fd, TDOUBLE, "RGSALPHA", &doublenull, dummy, &status))
      printerror( status );
  alp=doublenull;
  if (fits_read_key (fd2, TDOUBLE, "RGSALPHA", &doublenull, dummy, &status))
      printerror( status );
  if (alp != doublenull) {fprintf(stderr,"%s %s\n
      Source: %e Background: %e\n",message,dummy,alp,doublenull);terminate(8);}
  calp=cos(alp);

/*  determine dispersion order  */
  if (fits_read_key (fd, TLONG, "RFLORDER", &nrow, dummy, &status))
      printerror( status );
  m=nrow;
  ii=nrow;
  if (fits_read_key (fd2, TLONG, "RFLORDER", &nrow, dummy, &status))
      printerror( status );
  if (ii != nrow) {fprintf(stderr,"%s dispersion orders\n  
      Source: %d Background: %ld\n",message,ii,nrow);terminate(8);}

/* --------------------------------------- */
/*     End header information              */
/* --------------------------------------- */

  chan	  = (int   *) malloc (sizeof(int) * ndim);
  e_min   = (double *) malloc (sizeof(double) * ndim);
  e_max   = (double *) malloc (sizeof(double) * ndim);
  qual    = (int   *) malloc (sizeof(int) * ndim);
  bqual   = (int   *) malloc (sizeof(int) * ndim);
  rate    = (double *) malloc (sizeof(double) * ndim);
  brate   = (double *) malloc (sizeof(double) * ndim);
  staterr = (double *) malloc (sizeof(double) * ndim);
  berr    = (double *) malloc (sizeof(double) * ndim);
  dlam    = (double *) malloc (sizeof(double) * ndim);
  flwave  = (double *) malloc (sizeof(double) * ndim);
  flux    = (double *) malloc (sizeof(double) * ndim);
  aeff	  = (float *) malloc (sizeof(float) * ndim);

/* ------------------------------------------ */
/*    reading ascii file created by rgsfluxer */  
/*     for obtaining wavelengths and fluxes   */
/*     if not present, Aeff will be set 0     */
/* ------------------------------------------ */

/* if (NULL == (fda = fopen(apath, "r")))
    {
      fprintf(stderr,"%s: File not found; can do without!", apath);
      for (ii=0;ii<ndim;++i) {flwave[ii]=0.;flux[ii]=0.;}
    }
 else
    {
      ii=0;  
      while(fgets(line, BUFSIZ-1, fda))  
	  {  
   	    if (line[0] != '#' && line[0] != '!')  
	      {  
         	items = sscanf(line, "%lf %lf",&flwave[ii],&flux[ii]);  
		if (ii>0) dlam[ii]=flwave[ii]-flwave[ii-1];
        	++ii;  
	      }  
	  }  
      dlam[0]=dlam[1];
      fclose(fda);
    } */

/* ------------------------------------------ */
/*    reading rmffile if present	      */
/* ------------------------------------------ */

  if ( 0 == access(rmfpath, R_OK) )
   {
     if ( fits_open_file(&fd3, rmfpath, READONLY, &status) )
         printerror( status );

     if ( fits_movnam_hdu(fd3, BINARY_TBL, "EBOUNDS", 0, &status) )
        printerror( status );

     fits_read_col(fd3, TDOUBLE, 2, 1, 1, ndim, &doublenull, e_min,
                    &anynull, &status);
     fits_read_col(fd3, TDOUBLE, 3, 1, 1, ndim, &doublenull, e_max,
                    &anynull, &status);

     if ( fits_close_file(fd3, &status) )
         printerror( status );
     rmfflag=1;
   }

/* --------------------------------------- */
/*    read fits files                      */
/* --------------------------------------- */

  for (ii = 0; ii < 4; ii++)      /* allocate space for the column labels */
      ttype[ii] = (char *) malloc(FLEN_VALUE);  /* max label length = 69 */

      /* read the column names from the TTYPEn keywords */
      fits_read_keys_str(fd, "TTYPE", 1, 4, ttype, &nfound, &status);

/*      printf(" Row  %10s %10s %10s %10s\n", ttype[0], ttype[1], ttype[2], ttype[3]); */

      frow      = 1;
      felem     = 1;
      nelem     = ndim;
      intnull  = 0;
      doublenull = 0.;

      /*  read the columns */
      fits_read_col(fd, TINT, 1, frow, felem, nelem, &intnull,  chan,
                    &anynull, &status);
      fits_read_col(fd, TDOUBLE, 2, frow, felem, nelem, &doublenull,  rate,
                    &anynull, &status);
      fits_read_col(fd, TDOUBLE, 3, frow, felem, nelem, &doublenull, staterr,
                    &anynull, &status);
      fits_read_col(fd, TINT, 4, frow, felem, nelem, &intnull, qual,
                    &anynull, &status);
      fits_read_col(fd2, TINT, 4, frow, felem, nelem, &intnull, bqual,
                    &anynull, &status);
      fits_read_col(fd2, TDOUBLE, 2, frow, felem, nelem, &doublenull,  brate,
                    &anynull, &status);
      fits_read_col(fd2, TDOUBLE, 3, frow, felem, nelem, &doublenull, berr,
                    &anynull, &status);

    for (ii = 0; ii < 4; ii++)      /* free the memory for the column labels */
        free( ttype[ii] );

/*    for (ii = 0; ii < ndim; ii++) free( name[ii] ); */
	/* free the memory for the string column */

    if ( fits_close_file(fd, &status) )
         printerror( status );
    if ( fits_close_file(fd2, &status) )
         printerror( status ); 

/* --------------------------------------- */
/*  determine number of acceptable entries */
/*   and create index arry good            */
/* --------------------------------------- */
  nrow=0;
  for (ii=0;ii<ndim;++ii) if (qual[ii]==0 && bqual[ii]==0) ++nrow;
  good   = (int   *) malloc (sizeof(int) * nrow);
  i=0;
  for (ii=0;ii<ndim;++ii) if (qual[ii]==0 && bqual[ii]==0) {good[i]=ii;++i;}

  lnum   = (int   *) malloc (sizeof(int) * nrow);
  wave   = (float *) malloc (sizeof(float) * nrow);
  sp     = (float *) malloc (sizeof(float) * nrow);
  err    = (float *) malloc (sizeof(float) * nrow);
  bg     = (float *) malloc (sizeof(float) * nrow);
  erbg   = (float *) malloc (sizeof(float) * nrow);
  if (lnum == NULL || wave==NULL || sp==NULL || err==NULL || bg==NULL || erbg==NULL)
    {
      fprintf(stderr,"* Warning *: Out of memory\n");
      return(-1);
    }

/* --------------------------------------- */
/*     create cora compliant vectors       */
/* --------------------------------------- */

  for (ii=0; ii<nrow; ++ii)
   {
     i=good[ii];

     if (rmfflag == 1) wave[ii] = hc/e_min[i] + lamcor; else
      {
        bet=betref+dbet* chan[i];
        wave[ii] = d*fabs(cos(bet)-calp)/m + lamcor;
      }

/*     if (i!=ndim) wave[ii] = hc/e_min[i+1]; else wave[ii]=wave[ii-1]; */
/*     wave[ii] = flwave[i]; */

/* convert background-subtracted rates into total counts */
     dt = 1.;
     bg[ii]   = dt*brate[i];
     erbg[ii] = dt*berr[i];

     if (totflag == 0)
      {
       sp[ii]   = dt*rate[i] + bg[ii];
       err[ii]  = dt*sqrt(staterr[i]*staterr[i] +berr[i]*berr[i]);
      }
     else
      {
       sp[ii]   = dt*rate[i];
       err[ii]  = dt*staterr[i];
      }

/*   if (flux[i] !=0 ) aeff[ii] = rate[i]/flux[i]/dlam[i]; else aeff[ii]=0.; */
   }

  return (nrow);
}

/*  -------------------------------------------------------------------- */
 
void
make_aefffile (char *path)
{

  FILE *fd;
  fprintf(stderr,"Writing to %s\n",path);
  fd = fopen (path, "w");
  for (i=0;i<ndim;++i)
     fprintf(fd,"%f %f", wave[i], aeff[i]);
  fclose(fd);
return;

}

/*  -------------------------------------------------------------------- */

void
make_asciifile (char *path)
{
  FILE *fd;
 
fprintf(stderr,"Writing to %s\n",path);

sprintf(title,"#%s\n","wave[A] sp[cts] staterr[cts] bg[cts] errbg[cts]");
 
  fd = fopen (path, "w");
  fprintf(fd,"#  %f\n",exposure);
  for (i=0;i<ndim;++i)
     fprintf(fd,"%f %f %f %f %f\n"
           , wave[i], sp[i], err[i], bg[i], erbg[i]);
fclose(fd);
return;
 
} 

/*  -------------------------------------------------------------------- */

int main(int argc, char *argv[])
{

char rgsfile[MAXPATH];
char bgfile[MAXPATH];
char outfile[MAXPATH];
char rmffile[MAXPATH];
char totfile[MAXPATH];
char afile[MAXPATH];
char usage[MAXPATH];

strncpy (rgsfile, "rgs_net1o1.pha", MAXPATH);
strncpy (bgfile, "rgs_bkg1o1.pha", MAXPATH);
strncpy (outfile, "spectrum.dat", MAXPATH);
strncpy (rmffile, "rgs_net1o1.rmf", MAXPATH);
strncpy (totfile, "rgs_tot1o1.pha", MAXPATH);
strncpy (afile, "flux.txt", MAXPATH);

strncpy (usage, "Usage: cora_rgs [-n rgsfile] [-t totfile] [-b backgroundfile] [-o outfile] \n", MAXPATH);

  while (argc > 1 && argv[1][0] == '-')
    {
      switch (argv[1][1])
        {
        case 'n':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (rgsfile, argv[1], MAXPATH);
          break;
        case 'b':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (bgfile, argv[1], MAXPATH);
          break;
        case 'r':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (rmffile, argv[1], MAXTEXTL);
	  break;
        case 't':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (totfile, argv[1], MAXTEXTL);
          break;
        case 'o':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (outfile, argv[1], MAXTEXTL);
	  break;
        case 'a':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (afile, argv[1], MAXTEXTL);
          break;
        default:
          fprintf (stderr, usage);
	  fprintf (stderr, "Defaults: rgsfile=%s or totfile=%s, backgroundfile=%s, outfile=%s\n",rgsfile,totfile,bgfile,outfile);
	  exit(8);
          break;
        }
      ++argv;
      --argc;
    }

    ndim = read_rgs(rgsfile,bgfile,rmffile,totfile,afile);
    assert (ndim !=-1);

    make_asciifile(outfile);

return (0);  
}

#endif
