#! /bin/sh

die () {
  echo "ERROR: $@"
  exit 1
}

#-------------------------------------------------
#        THIS SECTION NEEDS TO BE EDITED  !!!    |
#-------------------------------------------------

# uncomment and modify these four variables for your system
#   location where SAS is installed in your system
#     (something like /usr/local/sas/xmmsas)
SAS_DIR=
#   location of your odf directory
SAS_ODF=
#   proposal ID (second set of number in filenames)
obs=
#   observation type (S for scheduled observation, U for unscheduled)
otype=S
#   unit numbers (from $SAS_ODF/*R1*AUX.FIT for expno1 located in odf directory)
#                      $SAS_ODF/R2*AUX.FIT for expno2
expno1=004
expno2=005
#   it is advised to indicate the coordinates
#    instead of relying on the pointing
#    set withsrc either y (yes) or n (no)
#    and give coordinates in decimal degrees
#     (srcdec=deg+min/60.+sec/3600.
#      srcra =15*(hrs+min/60.+sec/3600.)
withsrc=n
srcra=
srcdec=
#   set binning for response matrices
#    set to one if one need for wavelength scale
#    other wise set to 4000
ebins=1

#----------------------------------------------------
#     from here on no changes should be necessary   |
#----------------------------------------------------

[ "${SAS_DIR}x" != x ] || die "SAS_DIR not set; you must edit the script file"
export SAS_DIR
[ "${SAS_ODF}x" != x ] || die "SAS_ODF not set"
export SAS_ODF
[ "${obs}x" != x ] || die "obs not set"
export obs
did=P${obs}; export did
[ "${otype}x" != x ] || die "otype not set"
export otype
[ "${expno1}x" != x ] || die "expno1 not set"
export expno1
[ "${expno2}x" != x ] || die "expno2 not set"
export expno2
[ "${withsrc}x" != x ] || die "withsrc not set"
export withsrc
[ "${ebins}x" != x ] || die "ebins not set"
export ebins

pt=1
 if [ "x$withsrc" == "xy" ]
   then
   pt=3
   [ "${srcra}x" != x ] || die "srcra not set"
   [ "${srcdec}x" != x ] || die "srcdec not set"
 fi
export pt
export srcra
export srcdec

#------------odf preparation-----------
cd ${SAS_ODF}
chmod u+w *
chmod u+wx .
echo Changing small letters to capitals in ${SAS_ODF}
list=`ls *`; for ff in $list; do nn=`echo $ff | tr [:lower:] [:upper:]`; mv $ff $nn >/dev/null 2>&1; done
cd ..
[ -d ./mydata ] || mkdir mydata 
cd mydata

#----setup XMM_SAS--------------------
  SAS_PATH=$SAS_DIR; export SAS_PATH
  SAS_CCFPATH=$SAS_DIR/ccf; export SAS_CCFPATH
echo setting up xmmsas
  . $SAS_DIR/sas-setup.sh
  [ $? = 0 ] || die "$SAS_DIR/sas-setup.sh failed"


#----------calibration files---------------
if [ -f ccf.cif ]; then echo using $PWD/ccf.cif as SAS_CCF; else
 echo '  producing calibration file ccf.cif with cifbuild command...'
  cifbuild withccfpath=yes ccfpath=$SAS_DIR/ccf/ fullpath=yes
  [ $? = 0 ] || die "cifbuild failed"
 echo '  ...done'
fi

  SAS_CCF=${SAS_ODF}../mydata/ccf.cif; export SAS_CCF

#---------odfingest----------------

list=`ls $SAS_ODF/*.SAS`
if [ "${list}x" == x ]; then
 echo '  running odfingest...'
  odfingest outdir=$SAS_ODF odfdir=$SAS_ODF
  [ $? = 0 ] || die "odfingest failed"
 echo '  ...done'
fi
list=`ls $SAS_ODF/*.SAS`
 echo using $list as odf-file

#--------rgsproc----------------------
echo '  running rgsproc this will take a while, please be patient...'
  list=`ls GTI.FIT`
  if [ "${list}x" != x ]; then
    rgsproc srcra=${srcra} srcdec=${srcdec} withsrc=${withsrc} srclabel=mjsrc bkgcorrect=no auxgtitables=GTI.FIT withmlambdacolumn=yes -V 5 >& my_rgsproc_logfile; else rgsproc srcra=${srcra} srcdec=${srcdec} withsrc=${withsrc} srclabel=mjsrc bkgcorrect=no withmlambdacolumn=yes -V 5 >& my_rgsproc_logfile
  fi
  [ $? = 0 ] || die "rgsproc failed"
echo '  ...done'

#--------region and banana plot----------------------

#   R1  #
evselect table="${did}R1${otype}${expno1}EVENLI0000.FIT:EVENTS" withimageset=yes imageset='my_spatial1.fit' xcolumn='BETA_CORR' ycolumn='XDSP_CORR'

evselect table="${did}R1${otype}${expno1}EVENLI0000.FIT:EVENTS" withimageset=yes imageset='my_banana1.fit' xcolumn='BETA_CORR' ycolumn='PI' withyranges=yes yimagemin=0 yimagemax=3000 expression="region(${did}R1${otype}${expno1}SRCLI_0000.FIT:RGS1_SRC1_SPATIAL,BETA_CORR,XDSP_CORR)"

rgsimplot endispset='my_banana1.fit' spatialset='my_spatial1.fit' srcidlist='1' srclistset="${did}R1${otype}${expno1}SRCLI_0000.FIT" withendispregionsets=yes withendispset=yes withspatialregionsets=yes withspatialset=yes device=/cps plotfile=region_R1.ps

#   R1  #
evselect table="${did}R2${otype}${expno2}EVENLI0000.FIT:EVENTS" withimageset=yes imageset='my_spatial2.fit' xcolumn='BETA_CORR' ycolumn='XDSP_CORR'

evselect table="${did}R2${otype}${expno2}EVENLI0000.FIT:EVENTS" withimageset=yes imageset='my_banana2.fit' xcolumn='BETA_CORR' ycolumn='PI' withyranges=yes yimagemin=0 yimagemax=3000 expression="region(${did}R2${otype}${expno2}SRCLI_0000.FIT:RGS2_SRC1_SPATIAL,BETA_CORR,XDSP_CORR)"

rgsimplot endispset='my_banana2.fit' spatialset='my_spatial2.fit' srcidlist='1' srclistset="${did}R2${otype}${expno2}SRCLI_0000.FIT" withendispregionsets=yes withendispset=yes withspatialregionsets=yes withspatialset=yes device=/cps plotfile=region_R2.ps

#--------RGS lightcurve --------------------

evselect table=${did}R1${otype}${expno1}EVENLI0000.FIT withrateset=yes rateset=RGS1_RATE.FIT makeratecolumn=yes maketimecolumn=yes timebinsize=75 expression="(CCDNR==9)&&(REGION(${did}R1${otype}${expno1}SRCLI_0000.FIT:RGS1_BACKGROUND,BETA_CORR,XDSP_CORR))"

evselect table=${did}R2${otype}${expno2}EVENLI0000.FIT withrateset=yes rateset=RGS2_RATE.FIT makeratecolumn=yes maketimecolumn=yes timebinsize=75 expression="(CCDNR==9)&&(REGION(${did}R2${otype}${expno2}SRCLI_0000.FIT:RGS2_BACKGROUND,BETA_CORR,XDSP_CORR))"

#-------------------------

echo '...ready'
echo You will find the results in the directory
echo $pwd

